package ru.dbotthepony.mc.otm.storage;

import net.minecraft.MethodsReturnNonnullByDefault;
import ru.dbotthepony.mc.otm.core.Fraction;

import javax.annotation.ParametersAreNonnullByDefault;
import java.math.BigDecimal;
import java.util.Optional;

/**
 * Interface implements all methods required for storage system functioning and interaction
 * with your kind of items/objects/whatever.
 *
 * You can either implement this interface directly (hard dependency, discouraged, unless you do an addon to this storage system)
 * or create a record class (preferred), which encapsulate your storage object.
 */
@MethodsReturnNonnullByDefault
@ParametersAreNonnullByDefault
public interface IStorageStack {
	IStorageStack copy();

	void setCount(Fraction value);
	Fraction getCount();

	boolean isEmpty();

	/**
	 * @return max stack size for this stack object
	 *         Optional.empty() if unlimited (default)
	 */
	default Optional<Fraction> getMaxStackSize() {
		return Optional.empty();
	}

	/**
	 * @return Identity utilized to partition view table (if it has any).
	 *         Defaults to no partitioning; meaning performance will degrade much quicker than it should.
	 *         Good object is an object that will influence on sameItem() return result, making it return true.
	 *         It is strictly considered that if !this.itemIdentity().equals(other.itemIdentity());
	 *         then this.sameItem(other) will never return true.
	 *
	 *         If implemented, and storage is also partitioned properly, then performance will be flat equally to view table's performance.
	 *
	 *         Example: ItemStack#getItem()
	 */
	default Object partitionKey() {
		return Object.class;
	};

	/**
	 * @param other other object to compare; if not the same of this object type just return false
	 * @return boolean representing whenever internal state of this equals to other;
	 *         this include tags, mapping IDs, capabilities, excluding amount;
	 *         behavior is pretty much the same as ItemStack.isSameItemSameTags
	 */
	boolean sameItem(IStorageStack other);

	default void grow(Fraction amount) {
		setCount(getCount().plus(amount));
	}

	default void shrink(Fraction amount) {
		setCount(getCount().minus(amount));
	}
}
